@@ -50,7 +50,7 @@ def _readword(s, i):
     return j, w
 
 
-def _parsestring(s, i=0):
+def _parsestring(s, i):
     """Parse double-quoted string looking like ["hello \"one phrase\" world"]
     Called with the first quote already read.
     Upnp search term strings are double quoted, but we should not take them as recoll phrases.
@@ -128,11 +128,11 @@ def _searchClauses(out, neg, field, oper, words, phrases):
     return out
 
 
-def _makeSearchExp(out, v, field, oper, neg):
+def _makeSearchExp(out, v, fields, oper, neg):
     """Process data from an UPnP relExp triplet.
     Build a recoll search expression, given:
     v: terms or phrases out of _parseString.
-    field: which we may expand to multiple ORed values (e.g. title -> title or filename)
+    fields: one or several fields to be ORed (e.g. title -> title or filename)
     oper: :, = etc. "I" for ignore.
     neg: exclusion"""
 
@@ -157,10 +157,8 @@ def _makeSearchExp(out, v, field, oper, neg):
 
     # Special-case 'title' because we want to also match the file name. Possibly only for
     # directories, but there is no way to do inside a single query (see comment further).
-    if field == "title":
-        fields = (field, "filename")
-    else:
-        fields = (field,)
+    if isinstance(fields, str):
+        fields = (fields,)
 
     if len(fields) > 1:
         out.append(" (")
@@ -174,7 +172,7 @@ def _makeSearchExp(out, v, field, oper, neg):
         # anyway:
         # if i == 1: out.append(" AND mime:inode/directory")
         out.append(")")
-        if len(fields) == 2 and i == 0:
+        if len(fields) > 1 and i < len(fields) -1:
             if neg:
                 out.append(" AND ")
             else:
@@ -189,7 +187,13 @@ def _makeSearchExp(out, v, field, oper, neg):
 _fieldaliases = {
     "dc:creator": "upnp:artist",
 }
-
+# Some upnp fields need to be expanded to multiple recoll ones and result in an OR search
+# Note that upnp has no notion at all of "composer" or "conductor", only upnp:artist or dc:creator.
+# The "role" value is free text as far as I can see.
+_fieldexpansions = {
+    "upnp:artist": ("artist", "composer", "conductor", "author"),
+    "dc:title": ("title", "filename"),
+}
 
 # Upnp searches are made of relExps which are always (selector, operator, value), there are no unary
 # operators. The relExp-s can be joined with and/or and grouped with parentheses, but they are
@@ -202,6 +206,22 @@ _fieldaliases = {
 #
 # This is all quite approximative though, but simpler than a formal parser, and mostly works in
 # practise because we rely on recoll to deal with logical operators and parentheses.
+# UPnP search string grammar as from the standard:
+#    searchCrit ::= searchExp | asterisk
+#    searchExp  ::= relExp |
+#                   searchExp wChar+ logOp wChar+ searchExp |
+#                   '(' wChar* searchExp wChar* ')'
+#    logOp      ::= 'and' | 'or'
+#    relExp     ::= property wChar+ binOp wChar+ quotedVal |
+#                   property wChar+ existsOp wChar+ boolVal
+#    binOp      ::= relOp |stringOp
+#    relOp      ::= '=' | '!=' | '<' | '<=' | '>' | '>='
+#    stringOp   ::= 'contains' | 'doesNotContain' | 'derivedFrom'
+#    existsOp   ::= 'exists'
+#    boolVal    ::= 'true' | 'false'
+#    wChar      ::= space | hTab | lineFeed | vTab | formFeed | return
+#    property   ::= (* property name as defined in section 2.4 *)
+#    escapedQuote ::= (* double-quote escaped string as defined in section 2.3.1 *)
 def _upnpsearchtorecoll(s):
     uplog(f"_upnpsearchtorecoll:in: <{s}>")
 
@@ -240,8 +260,24 @@ def _upnpsearchtorecoll(s):
                 # Quoted strings are always values (3rd triplet elements) and values are always
                 # quoted. Read it, then generate and append one or several recoll search clauses,
                 # according to the already read 1st and 2nd triplet elements.
+
+                # We must have a field and an operator at this point.
+                if not field or not oper:
+                    uplog(f"Search: syntax error: [{s}]")
+                    return None
+
                 i, v = _parsestring(s, i)
+
+                # If the operator is "=" for a string field, we transform the search into an
+                # anchored phrase search.
+                if field != "mime" and oper == "=" :
+                    fs = "^" + " ".join(v) + "$"
+                    v = [fs]
+                    oper = ":"
+
                 _makeSearchExp(out, v, field, oper, neg)
+
+                # Reset everything and go on
                 field = ""
                 oper = ""
                 neg = False
@@ -281,7 +317,10 @@ def _upnpsearchtorecoll(s):
                     try:
                         if w in _fieldaliases:
                             w = _fieldaliases[w]
-                        field = uprclutils.upnp2rclfields[w]
+                        if w in _fieldexpansions:
+                            field = _fieldexpansions[w]
+                        else:
+                            field = uprclutils.upnp2rclfields[w]
                     except Exception as ex:
                         uplog(f"Field translation error for <{w}>: {ex}. Ignoring search.")
                         # Recoll would just ignore an unknown field spec and search the main
